package main

import (
	"bytes"
	"fmt"
	"time"

	"github.com/olekukonko/tablewriter"
)

type Item struct {
	Group         string    `yaml:"Group"`
	Description   string    `yaml:"Description"`
	DeliveryUntil time.Time `yaml:"Delivery until"`
	ProvidedOn    time.Time `yaml:"Provided on"`
	ProvidedBy    string    `yaml:"Provided by"`
}

func buildItemTable(config *Configuration, pageDataList []pageData) (error, string) {

	buf := new(bytes.Buffer)

	table := tablewriter.NewWriter(buf)

	table.SetHeader([]string{
		printer.Sprintf("Group"),
		printer.Sprintf("Description"),
		printer.Sprintf("Delivery until"),
		printer.Sprintf("Provided on"),
		printer.Sprintf("Provided by"),
		printer.Sprintf("File"),
	})
	table.SetBorders(tablewriter.Border{Left: true, Top: false, Right: true, Bottom: false})
	table.SetCenterSeparator("|")

	var tableData [][]string

	for _, pageData := range pageDataList {
		for _, info := range pageData.Items {

			var db, do string

			if info.DeliveryUntil.IsZero() {
				db = "—"
			} else {
				db = info.DeliveryUntil.Format(config.Locale.DateFormat)
			}
			if info.ProvidedOn.IsZero() {
				do = "—"
			} else {
				do = info.ProvidedOn.Format(config.Locale.DateFormat)
			}

			//text := info.Description + ", "
			//text += printer.Sprintf("Group") + ": " + info.Group + ", "
			//text += printer.Sprintf("File") + ": " + pageData.File

			tableData = append(tableData, []string{info.Group, info.Description, db, do, info.ProvidedBy, pageData.File})
			//tableData = append(tableData, []string{text, db, do, info.ProvidedBy})
		}

	}

	table.AppendBulk(tableData) // Add Bulk Data
	table.SetHeaderAlignment(tablewriter.ALIGN_LEFT)
	table.Render()

	return nil, buf.String()
}

func printItemTable(config *Configuration) error {
	err, pageData := GetPageData(config.Path)
	if err != nil {
		return err
	}

	err, table := buildItemTable(config, pageData)
	fmt.Println(table)

	return nil
}
