package main

import (
	"golang.org/x/text/language"
	"golang.org/x/text/message"
	"golang.org/x/text/message/catalog"
)

type l10nLocaleTranslation struct {
	tag string
	msg interface{}
}
type l10nKeyTranslation struct {
	key string
	tm  []l10nLocaleTranslation
}

var l10nMap = []l10nKeyTranslation{
	{
		"does not contain a valid yaml definition",
		[]l10nLocaleTranslation{
			{"de", "\"%s\" enthält keine gültige yaml Struktur."},
			{"en", "\"%s\" does not contain a valid yaml definition."},
		},
	},
	{
		"the specified template",
		[]l10nLocaleTranslation{
			{"de", "Das angegebene Template \"%s\" wurde nicht gefunden."},
			{"en", "the specified template \"%s\" was not found."},
		},
	},
	{
		"the due of the todo cannot be parsed",
		[]l10nLocaleTranslation{
			{"de", "Das Datum %s der Aufgabe \"%s\" kann nicht verarbeitet werden."},
			{"en", "the due %s of the todo \"%s\" cannot be parsed."},
		},
	},
	{
		"ToDo",
		[]l10nLocaleTranslation{
			{"de", "Aufgabe"},
			{"en", "task"},
		},
	},
	{
		"Due",
		[]l10nLocaleTranslation{
			{"de", "Bis"},
			{"en", "due"},
		},
	}, {
		"File",
		[]l10nLocaleTranslation{
			{"de", "Datei"},
			{"en", "file"},
		},
	}, {
		"Group",
		[]l10nLocaleTranslation{
			{"de", "Gruppe"},
			{"en", "Group"},
		},
	},
	{
		"Description",
		[]l10nLocaleTranslation{
			{"de", "Beschreibung"},
			{"en", "Description"},
		},
	},
	{
		"Delivery until",
		[]l10nLocaleTranslation{
			{"de", "Bis"},
			{"en", "until"},
		},
	},
	{
		"Provided on",
		[]l10nLocaleTranslation{
			{"de", "Am"},
			{"en", "on"},
		},
	},
	{
		"Provided by",
		[]l10nLocaleTranslation{
			{"de", "Von"},
			{"en", "by"},
		},
	},
	{
		"Type",
		[]l10nLocaleTranslation{
			{"de", "Typ"},
			{"en", "Type"},
		},
	},
	{
		"Title",
		[]l10nLocaleTranslation{
			{"de", "Titel"},
			{"en", "Title"},
		},
	},
	{
		"Alias",
		[]l10nLocaleTranslation{
			{"de", "Alias"},
			{"en", "Alias"},
		},
	},
	{
		"Type",
		[]l10nLocaleTranslation{
			{"de", "Typ"},
			{"en", "Type"},
		},
	},
	{
		"Keywords",
		[]l10nLocaleTranslation{
			{"de", "Schlüsselwörter"},
			{"en", "Keywords"},
		},
	},
	{
		"Author",
		[]l10nLocaleTranslation{
			{"de", "Autor"},
			{"en", "Author"},
		},
	},
	{
		"Status",
		[]l10nLocaleTranslation{
			{"de", "Status"},
			{"en", "Status"},
		},
	},
	{
		"Complexity",
		[]l10nLocaleTranslation{
			{"de", "Komplexität"},
			{"en", "Complexity"},
		},
	},
	{
		"Difficulty",
		[]l10nLocaleTranslation{
			{"de", "Schwierigkeit"},
			{"en", "Difficulty"},
		},
	},
	{
		"Priority",
		[]l10nLocaleTranslation{
			{"de", "Priorität"},
			{"en", "Priority"},
		},
	},
	{
		"Version",
		[]l10nLocaleTranslation{
			{"de", "Version"},
			{"en", "Version"},
		},
	},
	{
		"Milestone",
		[]l10nLocaleTranslation{
			{"de", "Meilenstein"},
			{"en", "Milestone"},
		},
	},
	{
		"Created",
		[]l10nLocaleTranslation{
			{"de", "Erstellt am"},
			{"en", "Created"},
		},
	},
	{
		"LastUpdate",
		[]l10nLocaleTranslation{
			{"de", "Letzte Änderung"},
			{"en", "LastUpdate"},
		},
	},
	{
		"The file cannot be read",
		[]l10nLocaleTranslation{
			{"de", "Die Datei \"%s\" kann nicht eingelesen werden."},
			{"en", "The file \"%s\" cannot be read."},
		},
	},
	{
		"The output file cannot be written",
		[]l10nLocaleTranslation{
			{"de", "Die Ausgabedatei kann nicht geschrieben werden (%s)."},
			{"en", "The output file cannot be written (%s)."},
		},
	}, {
		"A temporary file cannot be created",
		[]l10nLocaleTranslation{
			{"de", "Es kann keine temporäre Datei angelegt werden (%s)."},
			{"en", "A temporary file cannot be created (%s)."},
		},
	},
}

func initL10n() {
	for _, e := range l10nMap {
		key := e.key
		for _, f := range e.tm {
			tag := language.MustParse(f.tag)
			switch msg := f.msg.(type) {
			case string:
				message.SetString(tag, key, msg)
			case catalog.Message:
				message.Set(tag, key, msg)
			case []catalog.Message:
				message.Set(tag, key, msg...)
			}
		}
	}
}
