package main

import (
	"bytes"
	"fmt"
	"reflect"
	"time"

	"github.com/olekukonko/tablewriter"
)

func translateMetaHeaders(columns []string) []string {

	result := []string{}

	for _, column := range columns {
		result = append(result, printer.Sprintf(column))
	}

	return result

}

func buildMeta(config *Configuration, pageDataList []pageData) (error, string) {

	buf := new(bytes.Buffer)

	table := tablewriter.NewWriter(buf)

	translateMetaColumns := translateMetaHeaders(config.Table.Columns)

	table.SetHeader(translateMetaColumns)
	table.SetBorders(tablewriter.Border{Left: true, Top: false, Right: true, Bottom: false})
	table.SetCenterSeparator("|")

	var tableData [][]string

	for _, pd := range pageDataList {

		r := reflect.ValueOf(pd)

		var row []string
		found := false
		for _, name := range config.Table.Columns {

			field := reflect.Indirect(r).FieldByName(name)
			if field == (reflect.Value{}) {
				row = append(row, "")
				continue
			}

			t := field.Type()

			switch t.Name() {
			case "string":
				found = true
				row = append(row, field.String())
			case "Time":
				found = true
				t := field.Interface().(time.Time)
				if t.IsZero() {
					row = append(row, "—")
				} else {
					row = append(row, field.Interface().(time.Time).Format(config.Locale.DateFormat))
				}
			case "int":
				found = true
				row = append(row, field.String())
			case "float64":
				found = true
				row = append(row, field.String())
			case "nil":
				row = append(row, field.String())
			case "bool":
				found = true
				expr := field.Bool()
				if expr {
					row = append(row, "true")
				} else {
					row = append(row, "false")
				}

			default:
				row = append(row, "")
			}

		}

		if found {
			tableData = append(tableData, row)
		}

	}

	table.AppendBulk(tableData) // Add Bulk Data
	table.SetHeaderAlignment(tablewriter.ALIGN_LEFT)
	table.Render()

	return nil, buf.String()

}

func printMeta(config *Configuration) error {
	err, pageData := GetPageData(config.Path)
	if err != nil {
		return err
	}

	err, table := buildMeta(config, pageData)
	fmt.Println(table)
	return err
}
