package main

import (
	"html/template"
	"io/ioutil"
	"os"
	"strings"
	"time"
)

type Dataset struct {
	Config       *Configuration
	Meta         string
	Tasks        string
	Items        string
	Requirements string
	Created      time.Time
}

func PrintOverview(config *Configuration, arguments *commandLineOptions) error {

	err, pageData := GetPageData(config.Path)
	if err != nil {
		return err
	}

	err, meta := buildMeta(config, pageData)
	if err != nil {
		return err
	}

	err, tasks := buildToDosTable(config, pageData)
	if err != nil {
		return err
	}

	err, items := buildItemTable(config, pageData)
	if err != nil {
		return err
	}

	t, err := template.New("overview").Parse(config.Overview.Template)
	if err != nil {
		return err
	}

	requirements := ""

	for _, p := range pageData {
		requirements = requirements + getOverviewContent(p.Absolute)
	}

	d := new(Dataset)
	d.Meta = meta
	d.Tasks = tasks
	d.Config = config
	d.Items = items
	d.Requirements = requirements
	d.Created = time.Now()
	//d.pageData = pageData

	if arguments.Overview.Print.Output == "" {
		err = t.Execute(os.Stdout, d)
		if err != nil {
			return err
		}
	}

	outputName := arguments.Overview.Print.Output

	if arguments.Overview.Print.Format == "pdf" {
		file, err := ioutil.TempFile(os.TempDir(), "prefix")
		if err != nil {
			printErrorAndExit(2, "A temporary file cannot be created", err.Error())
		}
		defer os.Remove(file.Name())

		err = t.Execute(file, d)
		if err != nil {
			return err
		}

		convertToPDF(file.Name(), outputName)

	} else {

		file, err := os.Create(outputName)
		if err != nil {
			printErrorAndExit(2, "The output file cannot be written", err.Error())
		}

		// close fo on exit and check for its returned error
		defer func() {
			if err := file.Close(); err != nil {
				printErrorAndExit(2, "The output file cannot be written", err.Error())
			}
		}()

		err = t.Execute(file, d)
		if err != nil {
			return err
		}

	}

	return nil

}

func getOverviewContent(absolute string) string {

	content, err := os.ReadFile(absolute)
	if err != nil {
		printError("The file cannot be read", absolute)
		return ""
	}

	s := string(content)
	a := strings.Index(s, "\n---")
	if a == -1 {
		return s
	}

	s = s[a+4 : len(s)-1]
	b := strings.Index(s, "\n...")

	if b == -1 {
		b = strings.Index(s, "\n---")
	}

	if b > -1 {
		s = s[b+4 : len(s)-1]
	}

	return s
}
