package main

import (
	"bytes"
	"fmt"
	"regexp"
	"time"

	"github.com/olekukonko/tablewriter"
)

type toDo struct {
	due  time.Time
	text string
}

const dateLayout = "2006-01-02 15:04:05 GMT"

func newToDo(date string, text string) toDo {

	var d time.Time
	var err error

	if date != "" {
		d, err = time.Parse(dateLayout, date+" 00:00:00 GMT")
		if err != nil {
			printError("the due of the todo cannot be parsed", date, text)
		}
	}

	t := toDo{
		due:  d,
		text: text,
	}

	return t
}

func findToDos(content []byte) []toDo {

	toDos := []toDo{}

	todoRegEx := regexp.MustCompile(`@todo\s*(?P<due>[2][0-9]{3}-[0-9]{2}-[0-9]{2}\s*){0,1}(?P<text>[^\n]*)`)

	s := string(content)
	matches := todoRegEx.FindAllStringSubmatch(s, -1)
	if matches == nil {
		return []toDo{}
	}

	for _, match := range matches {
		result := make(map[string]string)
		for i, name := range todoRegEx.SubexpNames() {
			if i != 0 && name != "" {
				result[name] = match[i]
			}
		}

		t := newToDo(result["due"], result["text"])
		toDos = append(toDos, t)

	}

	return toDos
}

func buildToDosTable(config *Configuration, pageDataList []pageData) (error, string) {

	buf := new(bytes.Buffer)

	table := tablewriter.NewWriter(buf)

	table.SetHeader([]string{
		printer.Sprintf("ToDo"),
		printer.Sprintf("Due"),
		printer.Sprintf("File")})
	table.SetBorders(tablewriter.Border{Left: true, Top: false, Right: true, Bottom: false})
	table.SetCenterSeparator("|")

	var tableData [][]string

	for _, pageData := range pageDataList {
		for _, info := range pageData.ToDos {

			var d string

			if info.due.IsZero() {
				d = "—"
			} else {
				d = info.due.Format(config.Locale.DateFormat)
			}

			tableData = append(tableData, []string{info.text, d, pageData.File})
		}

	}

	table.AppendBulk(tableData) // Add Bulk Data
	table.SetHeaderAlignment(tablewriter.ALIGN_LEFT)
	table.Render()

	return nil, buf.String()
}

func printToDoTable(config *Configuration) error {
	err, pageData := GetPageData(config.Path)
	if err != nil {
		return err
	}

	err, table := buildToDosTable(config, pageData)
	fmt.Println(table)

	return nil
}
